@@ -147,12 +147,9 @@ class RawReader(object):
 class SeekableRawReader(object):
     """Read an S3 object."""
 
-    def __init__(self, s3_object):
+    def __init__(self, s3_object, content_length):
         self._object = s3_object
-        try:
-            self._content_length = self._object.content_length
-        except botocore.client.ClientError:
-            raise ValueError('the s3 key %r does not exist, or is forbidden for access' % s3_object.key)
+        self._content_length = content_length
         self.seek(0)
 
     def seek(self, position):
@@ -339,8 +336,16 @@ class SeekableBufferedInputBase(BufferedInputBase):
             resource_kwargs = {}
         s3 = session.resource('s3', **resource_kwargs)
         self._object = s3.Object(bucket, key)
-        self._raw_reader = SeekableRawReader(self._object)
-        self._content_length = self._object.content_length
+
+        try:
+            self._content_length = self._object.content_length
+        except botocore.client.ClientError:
+            raise ValueError(
+                '%r does not exist in the bucket %r, '
+                'or is forbidden for access' % (key, bucket)
+            )
+
+        self._raw_reader = SeekableRawReader(self._object, self._content_length)
         self._current_pos = 0
         self._buffer = smart_open.bytebuffer.ByteBuffer(buffer_size)
         self._eof = False
@@ -418,17 +423,12 @@ multipart upload may fail")
             multipart_upload_kwargs = {}
 
         s3 = session.resource('s3', **resource_kwargs)
-
-        #
-        # https://stackoverflow.com/questions/26871884/how-can-i-easily-determine-if-a-boto-3-s3-bucket-resource-exists
-        #
         try:
-            s3.meta.client.head_bucket(Bucket=bucket)
+            self._object = s3.Object(bucket, key)
+            self._min_part_size = min_part_size
+            self._mp = self._object.initiate_multipart_upload(**multipart_upload_kwargs)
         except botocore.client.ClientError:
             raise ValueError('the bucket %r does not exist, or is forbidden for access' % bucket)
-        self._object = s3.Object(bucket, key)
-        self._min_part_size = min_part_size
-        self._mp = self._object.initiate_multipart_upload(**multipart_upload_kwargs)
 
         self._buf = io.BytesIO()
         self._total_bytes = 0
